SystemOrganization addCategory: #'Polymorph-EventEnhancements'!

WeakMessageSend weakSubclass: #NonReentrantWeakMessageSend
	instanceVariableNames: 'executing'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-EventEnhancements'!

!NonReentrantWeakMessageSend commentStamp: 'LaurentLaffont 4/15/2011 20:18' prior: 0!
NonReentrantWeakMessageSend does what it says, while the message is being executed, additional evaluations will be discarded.
It's used by when:sendOnce:to: protocol from Polymorph events, but unused in the base image.
when:send:to:exclusive: is used one place though.

It's useful when objects are mutually registered to each other's events, but the actions may lead to the others action being triggered.
Examples are 
- two lists whose contents update based on the selection in the other
- The DiffMorph (Uses ExclusiveWeakMessageSend)

ExclusiveWeakMessageSend are used when the decision whether to process an event is shared between multiple objects.
IE 2 objects respond to different events, but if received simultaneously, only the first of them should have it's action executed.

With Announcement, the corresponding functionality to non-reentrancy would be achieved using:
VW - AnnouncementCollection>>suspendWhile:  anActionBlock
Pharo - Announcer >> suspend: aSubscriber while: anActionBlock (As we neither have a specific AnnouncementCollection class, nor access to Registry itself. Not implemented yet though :P)

The shared state required to achieve Exclusivity would probably have to recide outside of the framework.!

NonReentrantWeakMessageSend weakSubclass: #ExclusiveWeakMessageSend
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Polymorph-EventEnhancements'!

!ExclusiveWeakMessageSend commentStamp: 'LaurentLaffont 4/15/2011 20:18' prior: 0!
See NonReentrantWeakMessageSend!

!ExclusiveWeakMessageSend class methodsFor: 'as yet unclassified' stamp: 'gvc 10/25/2006 18:13'!
newSharedState
	"Answer a new ValueHolder with false as the contents."

	^ValueHolder new contents: false! !

!ExclusiveWeakMessageSend methodsFor: 'evaluating' stamp: 'gvc 7/30/2009 13:39'!
basicExecuting: aValueHolder
	"Set the shared value holder."

	executing := aValueHolder! !

!ExclusiveWeakMessageSend methodsFor: 'accessing' stamp: 'gvc 10/25/2006 18:07'!
executing
	"Answer from the shared value holder."

	^executing contents! !

!ExclusiveWeakMessageSend methodsFor: 'accessing' stamp: 'gvc 10/25/2006 18:06'!
executing: aBoolean
	"Set on the shared value holder."

	executing contents: aBoolean! !

!ExclusiveWeakMessageSend methodsFor: 'initialize-release' stamp: 'gvc 10/25/2006 18:13'!
initialize
	"Initialize the receiver."

	executing := self class newSharedState.
	super initialize.! !

!NonReentrantWeakMessageSend methodsFor: 'evaluating' stamp: 'gvc 10/25/2006 18:03'!
execute: aBlock
	"Answer the value of the block or nil if already executing."

	self executing ifTrue: [^nil].
	self executing: true.
	^aBlock ensure: [self executing: false]! !

!NonReentrantWeakMessageSend methodsFor: 'accessing' stamp: 'gvc 10/25/2006 18:03'!
executing
	"Answer the value of executing"

	^ executing! !

!NonReentrantWeakMessageSend methodsFor: 'accessing' stamp: 'gvc 10/25/2006 18:03'!
executing: anObject
	"Set the value of executing"

	executing := anObject! !

!NonReentrantWeakMessageSend methodsFor: 'initialize-release' stamp: 'gvc 10/25/2006 18:04'!
initialize
	"Initialize the receiver."

	super initialize.
	self executing: false! !

!NonReentrantWeakMessageSend methodsFor: 'testing' stamp: 'gvc 10/25/2006 18:15'!
isValid
	"Answer the superclass vlaue of isValid or false if
	executing is true."

	^self executing
		ifTrue: [false]
		ifFalse: [super isValid]! !

!NonReentrantWeakMessageSend methodsFor: 'evaluating' stamp: 'gvc 10/24/2006 11:42'!
value
	"Answer the superclass value or nil if already executing."

	^self execute: [super value]! !

!NonReentrantWeakMessageSend methodsFor: 'evaluating' stamp: 'gvc 10/24/2006 11:56'!
valueWithArguments: anArray
	"Answer the superclass value or nil if already executing."

	^self execute: [super valueWithArguments: anArray]! !

!NonReentrantWeakMessageSend methodsFor: 'evaluating' stamp: 'gvc 10/24/2006 11:56'!
valueWithEnoughArguments: anArray
	"Answer the superclass value or nil if already executing."

	^self execute: [super valueWithEnoughArguments: anArray]! !

!Object methodsFor: '*Polymorph-EventEnhancements' stamp: 'gvc 10/25/2006 18:18'!
when: anEventSelector
send: aMessageSelector
to: anObject
exclusive: aValueHolder
 
	self
		when: anEventSelector
		evaluate: ((ExclusiveWeakMessageSend
					receiver: anObject
					selector: aMessageSelector)
						basicExecuting: aValueHolder)! !

!Object methodsFor: '*Polymorph-EventEnhancements' stamp: 'gvc 10/25/2006 18:17'!
when: anEventSelector
send: aMessageSelector
to: anObject
with: anArg
exclusive: aValueHolder
 
    self
        when: anEventSelector
        evaluate: ((ExclusiveWeakMessageSend
 		receiver: anObject
		selector: aMessageSelector
		arguments: (Array with: anArg))
			basicExecuting: aValueHolder)! !

!Object methodsFor: '*Polymorph-EventEnhancements' stamp: 'gvc 10/25/2006 18:17'!
when: anEventSelector
send: aMessageSelector
to: anObject
withArguments: anArgArray
exclusive: aValueHolder
 
    self
        when: anEventSelector
        evaluate: ((ExclusiveWeakMessageSend
		receiver: anObject
		selector: aMessageSelector
		arguments: anArgArray)
			basicExecuting: aValueHolder)! !

!Object methodsFor: '*Polymorph-EventEnhancements' stamp: 'gvc 10/24/2006 11:50'!
when: anEventSelector
sendOnce: aMessageSelector
to: anObject
 
    self
        when: anEventSelector
        evaluate: (NonReentrantWeakMessageSend
            receiver: anObject
            selector: aMessageSelector)! !

!Object methodsFor: '*Polymorph-EventEnhancements' stamp: 'gvc 10/24/2006 11:49'!
when: anEventSelector
sendOnce: aMessageSelector
to: anObject
with: anArg
 
    self
        when: anEventSelector
        evaluate: (NonReentrantWeakMessageSend
            receiver: anObject
            selector: aMessageSelector
		arguments: (Array with: anArg))! !

!Object methodsFor: '*Polymorph-EventEnhancements' stamp: 'gvc 10/24/2006 11:49'!
when: anEventSelector
sendOnce: aMessageSelector
to: anObject
withArguments: anArgArray
 
    self
        when: anEventSelector
        evaluate: (NonReentrantWeakMessageSend
            receiver: anObject
            selector: aMessageSelector
		arguments: anArgArray)! !

!SequenceableCollection methodsFor: '*Polymorph-EventEnhancements' stamp: 'nice 1/5/2010 15:59'!
aggregateRuns: aBlock
	"Answer a new collection of the same species as the
	receiver with elements being collections (of the receiver
	species) containing those elements of the receiver
	for which the given block consecutively evaluates to
	the same object."

	| str eStr r|
	str := Array new writeStream.
	r := nil.
	eStr := Array new writeStream.
	self do: [:e | | t |
		(t := aBlock value: e) = r
			ifTrue: [eStr nextPut: e]
			ifFalse: [r := t.
					eStr isEmpty
						ifFalse: [str nextPut: (eStr contents as: self species).
								eStr reset].
					eStr nextPut: e]].
	eStr isEmpty ifFalse: [str nextPut: (eStr contents as: self species)].
	^str contents as: self species
	! !
